#include "DirectoryWatcher.h"
#include <stdio.h>
#include <unistd.h>
#include <String.h>

DirectoryWatcher::DirectoryWatcher(sem_id blocker, BDirectory &toWatch, BEntry &forDir,
				char *move, char *stat, char *attribute, char *deleted,
				char *newfile, char *removefile)
 : FileWatcher(blocker, forDir, move, stat, attribute, deleted),
   DirectoryMonitor(toWatch), PathMonitor(forDir) {
	mNewFile = newfile;
	mRemovedFile = removefile;
}

DirectoryWatcher::~DirectoryWatcher() {

}


void DirectoryWatcher::EntryCreated(const char *created_name, ino_t created_node) {
	BString command(mNewFile);
	command.ReplaceAll("%%", "\1\1");
	
	command.ReplaceAll("%1", mPath.Path());
	command.ReplaceAll("%2", created_name);
	
	command.ReplaceAll("\1\1", "%%");
	
	if (sNodeMonitorDebugging) {
		printf("Executing new-file command: %s\n", command.String());
	}

	system(command.String());
	CheckForQuit();
}

void DirectoryWatcher::EntryMovedIn(node_ref &from_directory, const char *name_of_node, ino_t node) {
	BString command(mNewFile);
	command.ReplaceAll("%%", "\1\1");
	
	command.ReplaceAll("%1", mPath.Path());
	command.ReplaceAll("%2", name_of_node);
	
	command.ReplaceAll("\1\1", "%%");
	
	if (sNodeMonitorDebugging) {
		printf("Executing new-file command: %s\n", command.String());
	}

	system(command.String());
	CheckForQuit();
}

void DirectoryWatcher::EntryMovedOut(node_ref &to_directory, const char *name_of_node, ino_t node) {
	BString command(mRemovedFile);
	command.ReplaceAll("%%", "\1\1");
	
	command.ReplaceAll("%1", mPath.Path());
	
	command.ReplaceAll("\1\1", "%%");
	
	if (sNodeMonitorDebugging) {
		printf("Executing remove-file command: %s\n", command.String());
	}

	system(command.String());
	CheckForQuit();
}

void DirectoryWatcher::EntryRemoved(ino_t node) {
	BString command(mRemovedFile);
	command.ReplaceAll("%%", "\1\1");
	
	command.ReplaceAll("%1", mPath.Path());
	
	command.ReplaceAll("\1\1", "%%");
	
	if (sNodeMonitorDebugging) {
		printf("Executing remove-file command: %s\n", command.String());
	}

	system(command.String());
	CheckForQuit();
}
