#include "FileWatcher.h"
#include "watcher.h"

#include <stdio.h>
#include <String.h>
#include <unistd.h>


FileWatcher::FileWatcher(sem_id blocker, BEntry &toWatch,
				char *move, char *stat, char *attribute, char *deleted)
 : PathMonitor(toWatch), mToWatch(toWatch) {

	mBlocker = blocker;
	mMove = move;
	mStat = stat;
	mAttribute = attribute;
	mDeleted = deleted;
}

FileWatcher::~FileWatcher() {
	release_sem (mBlocker);
}


void FileWatcher::PathChanged(BPath &oldPath) {
	BString command(mMove);
	command.ReplaceAll("%%", "\1\1");
	
	command.ReplaceAll("%1", oldPath.Path());
	command.ReplaceAll("%2", mPath.Path());
	
	command.ReplaceAll("\1\1", "%%");
	
	if (sNodeMonitorDebugging) {
		printf("Executing move command: %s\n", command.String());
	}

	system(command.String());
	CheckForQuit();
}

void FileWatcher::EntryDeleted(node_ref &fromDirectory) {
	BString command(mDeleted);
	command.ReplaceAll("%%", "\1\1");
	
	command.ReplaceAll("%1", mPath.Path());
	
	command.ReplaceAll("\1\1", "%%");
	
	if (sNodeMonitorDebugging) {
		printf("Executing delete command: %s\nTerminating watch.\n", command.String());
	}

	system(command.String());
	
	delete this;
}

void FileWatcher::StatChanged() {
	BString command(mStat);
	command.ReplaceAll("%%", "\1\1");
	
	command.ReplaceAll("%1", mPath.Path());
	
	command.ReplaceAll("\1\1", "%%");
	
	if (sNodeMonitorDebugging) {
		printf("Executing stat-changed command: %s\n", command.String());
	}

	system(command.String());
	CheckForQuit();
}

void FileWatcher::AttributeChanged(const char *which) {
	BString command(mAttribute);
	command.ReplaceAll("%%", "\1\1");
	
	command.ReplaceAll("%1", mPath.Path());
	command.ReplaceAll("%2", which);
	
	command.ReplaceAll("\1\1", "%%");
	
	if (sNodeMonitorDebugging) {
		printf("Executing attribute command: %s\n", command.String());
	}

	system(command.String());
	CheckForQuit();
}

void FileWatcher::CheckForQuit() {
	if (sDoOnlyOne)
		delete this;
}
